@@ -1,16 +1,17 @@
 /*
- * Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
+ * This program is designed to work with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms, as
  * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -18,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #pragma once
@@ -413,6 +414,10 @@ namespace wb {
 
     grt::ListRef<app_PaperType> get_paper_types(std::shared_ptr<grt::internal::Unserializer> unserializer);
 
+    std::vector<grt::SlotHolder*> _messageHandlerList;
+
+    void pushMessageHandler(grt::SlotHolder *slot);
+
     bool _other_connections_loaded;
     // setup
     void init_templates();
@@ -498,9 +503,6 @@ namespace wb {
     std::function<void(std::string, void *)> show_gui_plugin;
 
   private:
-    std::shared_ptr<grt::GRT> _grt; // Keep a local reference to the singleton to avoid static fiasco.
-    std::shared_ptr<bec::GRTManager> _grtManager;
-
     void warnIfRunningOnUnsupportedOS();
   };
 
