@@ -1,16 +1,17 @@
 /*
- * Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
+ * This program is designed to work with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms, as
  * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -37,241 +38,6 @@
 
 DEFAULT_LOG_DOMAIN(DOMAIN_BASE);
 
-// updated as of 5.7
-static const char *reserved_keywords[] = {"ACCESSIBLE",
-                                          "ADD",
-                                          "ALL",
-                                          "ALTER",
-                                          "ANALYZE",
-                                          "AND",
-                                          "AS",
-                                          "ASC",
-                                          "ASENSITIVE",
-                                          "BEFORE",
-                                          "BETWEEN",
-                                          "BIGINT",
-                                          "BINARY",
-                                          "BLOB",
-                                          "BOTH",
-                                          "BY",
-                                          "CALL",
-                                          "CASCADE",
-                                          "CASE",
-                                          "CHANGE",
-                                          "CHAR",
-                                          "CHARACTER",
-                                          "CHECK",
-                                          "COLLATE",
-                                          "COLUMN",
-                                          "CONDITION",
-                                          "CONSTRAINT",
-                                          "CONTINUE",
-                                          "CONVERT",
-                                          "CREATE",
-                                          "CROSS",
-                                          "CURRENT_DATE",
-                                          "CURRENT_TIME",
-                                          "CURRENT_TIMESTAMP",
-                                          "CURRENT_USER",
-                                          "CURSOR",
-                                          "DATABASE",
-                                          "DATABASES",
-                                          "DAY_HOUR",
-                                          "DAY_MICROSECOND",
-                                          "DAY_MINUTE",
-                                          "DAY_SECOND",
-                                          "DEC",
-                                          "DECIMAL",
-                                          "DECLARE",
-                                          "DEFAULT",
-                                          "DELAYED",
-                                          "DELETE",
-                                          "DESC",
-                                          "DESCRIBE",
-                                          "DETERMINISTIC",
-                                          "DISTINCT",
-                                          "DISTINCTROW",
-                                          "DIV",
-                                          "DOUBLE",
-                                          "DROP",
-                                          "DUAL",
-                                          "EACH",
-                                          "ELSE",
-                                          "ELSEIF",
-                                          "ENCLOSED",
-                                          "ESCAPED",
-                                          "EXISTS",
-                                          "EXIT",
-                                          "EXPLAIN",
-                                          "FALSE",
-                                          "FETCH",
-                                          "FLOAT",
-                                          "FLOAT4",
-                                          "FLOAT8",
-                                          "FOR",
-                                          "FORCE",
-                                          "FOREIGN",
-                                          "FROM",
-                                          "FULLTEXT",
-                                          "GET",
-                                          "GRANT",
-                                          "GROUP",
-                                          "HAVING",
-                                          "HIGH_PRIORITY",
-                                          "HOUR_MICROSECOND",
-                                          "HOUR_MINUTE",
-                                          "HOUR_SECOND",
-                                          "IF",
-                                          "IGNORE",
-                                          "IN",
-                                          "INDEX",
-                                          "INFILE",
-                                          "INNER",
-                                          "INOUT",
-                                          "INSENSITIVE",
-                                          "INSERT",
-                                          "INT",
-                                          "INT1",
-                                          "INT2",
-                                          "INT3",
-                                          "INT4",
-                                          "INT8",
-                                          "INTEGER",
-                                          "INTERVAL",
-                                          "INTO",
-                                          "IO_AFTER_GTIDS",
-                                          "IO_BEFORE_GTIDS",
-                                          "IS",
-                                          "ITERATE",
-                                          "JOIN",
-                                          "KEY",
-                                          "KEYS",
-                                          "KILL",
-                                          "LEADING",
-                                          "LEAVE",
-                                          "LEFT",
-                                          "LIKE",
-                                          "LIMIT",
-                                          "LINEAR",
-                                          "LINES",
-                                          "LOAD",
-                                          "LOCALTIME",
-                                          "LOCALTIMESTAMP",
-                                          "LOCK",
-                                          "LONG",
-                                          "LONGBLOB",
-                                          "LONGTEXT",
-                                          "LOOP",
-                                          "LOW_PRIORITY",
-                                          "MASTER_BIND",
-                                          "MASTER_SSL_VERIFY_SERVER_CERT",
-                                          "MATCH",
-                                          "MAXVALUE",
-                                          "MEDIUMBLOB",
-                                          "MEDIUMINT",
-                                          "MEDIUMTEXT",
-                                          "MIDDLEINT",
-                                          "MINUTE_MICROSECOND",
-                                          "MINUTE_SECOND",
-                                          "MOD",
-                                          "MODIFIES",
-                                          "NATURAL",
-                                          "NONBLOCKING",
-                                          "NOT",
-                                          "NO_WRITE_TO_BINLOG",
-                                          "NULL",
-                                          "NUMERIC",
-                                          "ON",
-                                          "OPTIMIZE",
-                                          "OPTION",
-                                          "OPTIONALLY",
-                                          "OR",
-                                          "ORDER",
-                                          "OUT",
-                                          "OUTER",
-                                          "OUTFILE",
-                                          "PARTITION",
-                                          "PRECISION",
-                                          "PRIMARY",
-                                          "PROCEDURE",
-                                          "PURGE",
-                                          "RANGE",
-                                          "READ",
-                                          "READS",
-                                          "READ_WRITE",
-                                          "REAL",
-                                          "REFERENCES",
-                                          "REGEXP",
-                                          "RELEASE",
-                                          "RENAME",
-                                          "REPEAT",
-                                          "REPLACE",
-                                          "REQUIRE",
-                                          "RESIGNAL",
-                                          "RESTRICT",
-                                          "RETURN",
-                                          "REVOKE",
-                                          "RIGHT",
-                                          "RLIKE",
-                                          "SCHEMA",
-                                          "SCHEMAS",
-                                          "SECOND_MICROSECOND",
-                                          "SELECT",
-                                          "SENSITIVE",
-                                          "SEPARATOR",
-                                          "SET",
-                                          "SHOW",
-                                          "SIGNAL",
-                                          "SMALLINT",
-                                          "SPATIAL",
-                                          "SPECIFIC",
-                                          "SQL",
-                                          "SQLEXCEPTION",
-                                          "SQLSTATE",
-                                          "SQLWARNING",
-                                          "SQL_BIG_RESULT",
-                                          "SQL_CALC_FOUND_ROWS",
-                                          "SQL_SMALL_RESULT",
-                                          "SSL",
-                                          "STARTING",
-                                          "STRAIGHT_JOIN",
-                                          "TABLE",
-                                          "TERMINATED",
-                                          "THEN",
-                                          "TINYBLOB",
-                                          "TINYINT",
-                                          "TINYTEXT",
-                                          "TO",
-                                          "TRAILING",
-                                          "TRIGGER",
-                                          "TRUE",
-                                          "UNDO",
-                                          "UNION",
-                                          "UNIQUE",
-                                          "UNLOCK",
-                                          "UNSIGNED",
-                                          "UPDATE",
-                                          "USAGE",
-                                          "USE",
-                                          "USING",
-                                          "UTC_DATE",
-                                          "UTC_TIME",
-                                          "UTC_TIMESTAMP",
-                                          "VALUES",
-                                          "VARBINARY",
-                                          "VARCHAR",
-                                          "VARCHARACTER",
-                                          "VARYING",
-                                          "WHEN",
-                                          "WHERE",
-                                          "WHILE",
-                                          "WITH",
-                                          "WRITE",
-                                          "XOR",
-                                          "YEAR_MONTH",
-                                          "ZEROFILL",
-                                          NULL};
-
 namespace base {
 
 #ifdef _MSC_VER
@@ -280,8 +46,8 @@ namespace base {
 
   //--------------------------------------------------------------------------------------------------
 
-  static std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>> utf16Converter;
-  static std::wstring_convert<std::codecvt_utf8<__int32>, __int32> utf32Converter;
+  thread_local static std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>> utf16Converter;
+  thread_local static std::wstring_convert<std::codecvt_utf8<__int32>, __int32> utf32Converter;
 
   /**
    * Converts an UTF-8 encoded string to an UTF-16 string.
@@ -1501,11 +1267,9 @@ namespace base {
 
   /**
    * Quotes the given identifier, but only if it needs to be quoted.
-   * http://dev.mysql.com/doc/refman/5.1/en/identifiers.html specifies what is allowed in unquoted identifiers.
-   * Leading numbers are not strictly forbidden but discouraged as they may lead to ambiguous behavior.
    */
-  std::string quote_identifier_if_needed(const std::string &ident, const char quote_char) {
-    bool needs_quotation = is_reserved_word(ident); // check whether it's a reserved keyword
+  std::string quoteIdentifierIfNeeded(const std::string &ident, const char quote_char, MySQLVersion version) {
+    bool needs_quotation = MySQLSymbolInfo::isReservedKeyword(ident, version);
     size_t digits = 0;
 
     if (!needs_quotation) {
@@ -1666,17 +1430,6 @@ namespace base {
 
   //--------------------------------------------------------------------------------------------------
 
-  bool is_reserved_word(const std::string &word) {
-    std::string upper = base::toupper(word);
-    for (const char **kw = reserved_keywords; *kw != NULL; ++kw) {
-      if (upper.compare(*kw) == 0)
-        return true;
-    }
-    return false;
-  }
-
-  //--------------------------------------------------------------------------------------------------
-
   EolHelpers::Eol_format EolHelpers::detect(const std::string &text) {
     std::string::size_type pos = text.find_first_of("\r\n");
     if (std::string::npos == pos)
