@@ -1,19 +1,24 @@
-# Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2 of the
-# License.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# This program is designed to work with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms, as
+# designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have either included with
+# the program or referenced in the documentation.
+# This program is distributed in the hope that it will be useful,  but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+# the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-# 02110-1301  USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 import locale
 import threading
@@ -22,7 +27,7 @@ from collections import deque
 import grt
 import mforms
 
-import wizard_page_widget
+from . import wizard_page_widget
 
 os_icon_suffix = ""
 import sys
@@ -35,7 +40,7 @@ class WizardTaskError(Exception):
 
 
 class WizardTask(mforms.Box):
-    def __init__(self, owner, func, label):
+    def __init__(self, owner, func, label, accessibilityName):
         mforms.Box.__init__(self, True)
         self.set_managed()
         self.set_release_on_add()
@@ -46,6 +51,7 @@ class WizardTask(mforms.Box):
         
         self._icon = mforms.newImageBox()
         self._label = mforms.newLabel(label)
+        self._label.set_name(accessibilityName)
         
         self._enabled = True
         
@@ -68,9 +74,9 @@ class WizardTask(mforms.Box):
             self.owner.send_info("%s cancelled" % self.label)
             self.set_aborted()
             return
-        except Exception, e:
+        except Exception as e:
             if not isinstance(self, WizardThreadedTask):
-                print
+                print()
                 import traceback
                 traceback.print_exc()
 
@@ -157,8 +163,8 @@ class WizardTask(mforms.Box):
 
 
 class WizardThreadedTask(WizardTask):
-    def __init__(self, owner, func, label):
-        WizardTask.__init__(self, owner, func, label)
+    def __init__(self, owner, func, label, accessibilityName):
+        WizardTask.__init__(self, owner, func, label, accessibilityName)
   
         self._thread = None
         
@@ -194,8 +200,8 @@ class WizardThreadedTask(WizardTask):
             self.owner.send_info("%s cancelled" % self.label)
             mforms.Utilities.perform_from_main_thread(self.set_aborted, False)
             return
-        except Exception, exc:
-            print
+        except Exception as exc:
+            print()
             import traceback
             traceback.print_exc()
             grt.log_error("Wizard", "Thread '%s' failed: %s\n" % (self.label, exc))
@@ -215,6 +221,7 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
         self._use_private_message_handling = use_private_message_handling
       
         self._description = mforms.newLabel(description or "The following tasks will now be performed. Please monitor the execution.")
+        self._description.set_name('Page Description')
         self.content.add(self._description, False, True)
         
         self._tasks_box = mforms.newBox(False)
@@ -222,7 +229,11 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
         self._tasks_box.set_spacing(8)
         self.content.add(self._tasks_box, False, True)
         
+        self._after_tasks_box = mforms.newBox(False)
+        self.content.add(self._after_tasks_box, False, True)
+        
         self._status_label = mforms.newLabel("Click [Next >] to execute.")
+        self._status_label.set_name("Next Information")
         self.content.add(self._status_label, False, True)
         
         self._progress = mforms.newProgressBar()
@@ -230,8 +241,10 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
         self._progress.show(False)
         
         self._detail_label = mforms.newLabel("")
+        self._detail_label.set_name('Details')
         self.content.add(self._detail_label, False, True)
-      
+
+        self._timer = None
         self._tasks = []
         self._currently_running_task_index = None
         self._progress_indeterminate = False
@@ -239,20 +252,24 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
         
         self._log_box = mforms.newPanel(mforms.TitledBoxPanel)
         self._log_box.set_title("Message Log")
-        self._log_box.set_padding(12)
+        self._log_box.set_name("Message Log")
         
         self._log_text = mforms.newTextBox(mforms.VerticalScrollBar)
+        self._log_text.set_name('Wizard Progress Log Text')
         self._log_text.set_read_only(True)
+        self._log_text.set_padding(16)
         self._log_box.add(self._log_text)
         self._log_box.show(False)
-        self.content.add(self._log_box, True, True)
+        self.content.add_end(self._log_box, True, True)
         self._log_queue = deque()
         
         self._showing_logs = False
         self._cancel_requested = False
+        self._tasks_held = False
         self.advanced_button.set_text("Show Logs")
         
         self._log_progress_text = True
+        self._autostart = False
 
     
     def clear_tasks(self):
@@ -264,7 +281,7 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
     def format_exception_text(self, e):
         return str(e)    
         
-    def add_task(self, func, label):
+    def add_task(self, func, label, accessibilityName):
         """ Add a task to be executed from the main thread. The callback must be as in
           callback(progress), where progress is a function to be optionally executed to
           report progress. 
@@ -273,13 +290,13 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
           by the user. If an error occurs during the execution of the task, an exception must 
           be raised.
         """
-        wtask = WizardTask(self, func, label)
+        wtask = WizardTask(self, func, label, accessibilityName)
         self._tasks.append(wtask)
         self._tasks_box.add(wtask, False, True)
         return wtask
     
     
-    def add_threaded_task(self, func, label):
+    def add_threaded_task(self, func, label, accessibilityName):
         """ Add a task to be executed from a worker thread. The callback must be as in
           callback(). Progress information must be reported using grt.send_progress()
            
@@ -287,7 +304,7 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
           by the user. If an error occurs during the execution of the task, an exception must 
           be raised.
         """
-        wtask = WizardThreadedTask(self, func, label)
+        wtask = WizardThreadedTask(self, func, label, accessibilityName)
         self._tasks.append(wtask)
         self._tasks_box.add(wtask, False, True)
         return wtask
@@ -298,8 +315,11 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
 
 
     def go_next(self):
-        if self._currently_running_task_index is None and not self._tasks_finished:
-            self.start()
+        if not self._tasks_finished:
+            if self._currently_running_task_index is None:
+                self.start()
+            else:
+                self.resume()
         else:
             wizard_page_widget.WizardPage.go_next(self)
 
@@ -310,6 +330,7 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
         else:
             self.advanced_button.set_text("Show Logs")
         self._log_box.show(self._showing_logs)
+        self.relayout()
     
     def go_cancel(self):
         self._cancel_requested = True
@@ -318,8 +339,8 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
         super(WizardProgressPage, self).page_activated(advancing)
         
         self.next_button.set_enabled(self._tasks_finished or self._currently_running_task_index is None)
-        
-        if advancing:
+
+        if advancing and self._autostart:
             if self._currently_running_task_index is None and not self._tasks_finished:
                 self.start()
 
@@ -355,15 +376,15 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
         self._log_queue.append(("OUTPUT", msg, ""))
 
     def send_info(self, msg):
-        grt.log_debug("Wizard", msg)
+        grt.log_debug("Wizard", msg + "\n")
         self._handle_task_output("INFO", msg, "")
         
     def send_error(self, msg):
-        grt.log_debug("Wizard", "ERROR: "+msg)
+        grt.log_debug("Wizard", "ERROR: %s\n" % msg )
         self._handle_task_output("ERROR", msg, "")
         
     def send_warning(self, msg):
-        grt.log_debug("Wizard", "WARNING: "+msg)
+        grt.log_debug("Wizard", "WARNING: %s\n" % msg )
         self._handle_task_output("WARNING", msg, "")
 
     def send_progress(self, pct, msg):
@@ -386,8 +407,24 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
             grt.push_status_query_handler(self.query_cancel_status)
             grt.push_message_handler(self._handle_task_output)
         self.send_info("Starting...")
-        mforms.Utilities.add_timeout(0.1, self.update_status)
+        self._timer = mforms.Utilities.add_timeout(0.1, self.update_status)
+        self.relayout()
         
+    def resume(self):
+        self._progress.show()
+        self.next_button.set_enabled(False)
+        self.back_button.set_enabled(False)
+        self.cancel_button.set_enabled(True)
+        
+        if not self._use_private_message_handling:
+            grt.push_status_query_handler(self.query_cancel_status)
+            grt.push_message_handler(self._handle_task_output)
+        self.send_info("Resuming...")
+
+        self._tasks_held = False
+        self._timer = mforms.Utilities.add_timeout(0.1, self.update_status)
+        self.relayout()
+
         
     def tasks_finished(self):
         pass
@@ -396,10 +433,26 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
     def tasks_failed(self, canceled):
         pass
         
+    def _hold(self):
+        self.send_info("Processing Held")
+        self._flush_messages()
+        self.send_info("\n")
+
+        self._progress.show(False)
+        self._progress.stop()
+        self._progress_indeterminate = False
+        self.next_button.set_enabled(True)
+        self.back_button.set_enabled(True)
+        self.cancel_button.set_enabled(True)
+
+        self._tasks_held = True
+        self.relayout()
+
+
 
     def _finished(self):
         if self._warnings > 0 or self._errors > 0:
-            self.send_info("Tasks finished with warnings and/or errors, view the logs for details")
+            self.send_info("Tasks finished with warnings and/or errors; view the logs for details")
         self.send_info(self.final_message())
         self._flush_messages()
         self.send_info("\n\n")
@@ -426,6 +479,7 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
             self.tasks_failed(False)
         else:
             self.tasks_finished()
+        self.relayout()
 
 
     def _failed(self):
@@ -449,6 +503,7 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
             self.go_advanced()
 
         self.tasks_failed(False)
+        self.relayout()
         
 
     def _cancelled(self):
@@ -469,6 +524,7 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
         self._detail_label.set_text("Cancelled by user.")
 
         self.tasks_failed(True)
+        self.relayout()
 
 
     def _flush_messages(self):
@@ -492,7 +548,7 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
                 try:
                     self.update_progress(locale.atof(pct), text)
                 except Exception:
-                    grt.log_debug('Wizard', 'Exception raised when converting "%s" to float using locale.atof(). Exception ignored' % pct)
+                    grt.log_debug('Wizard', 'Exception raised when converting "%s" to float using locale.atof(). Exception ignored\n' % pct)
             else:
                 if mtype == "ERROR":
                     self._errors += 1
@@ -519,17 +575,26 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
         """Subclass and override to change the text message to be shown when tasks finish successfully."""
         ret_val = ""
         if self._errors > 0:
-            ret_val = "There were errors during execution, please review log messages."
+            ret_val = "There were errors during execution; please review log messages."
         elif self._warnings > 0:
-            ret_val = "There were warnings during execution, please review log messages.\nClick [Next >] to continue if you think they are not important."
+            ret_val = "There were warnings during execution; please review log messages.\nClick [Next >] to continue if you think they are not important."
         else:
-            ret_val = self.final_message() + "\nClick [Next >] to continue."
+            ret_val = self.final_message() + " Click [Next >] to continue."
 
         return ret_val
 
+    def verify_task_preconditions(self, label):
+        """
+        Will check for specific task conditions to be met, task will be identified by its label.
+        If preconditions are met returns None, if not, returns a descriptive error message.
+        """
+        return None
 
     def update_status(self):
         self._flush_messages()
+
+        if self._tasks_held:
+            return False
     
         if self._progress_info:
             pct, text = self._progress_info
@@ -555,8 +620,15 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
                 return False
 
             try:
-                task.run()
-            except Exception, exc:
+                error = self.verify_task_preconditions(task.label)
+                if error is None:
+                    task.run()
+                else:
+                    self._hold()
+                    mforms.Utilities.show_error("Unable To Continue", error, "OK", "", "")
+
+
+            except Exception as exc:
                 #print
                 #import traceback
                 #traceback.print_exc()
